/* $Id: config.h,v 1.1.1.1 2004/10/11 17:35:05 mjhammel Exp $ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2001
 *========================================================================
 *              FILE NAME: config.h
 *            DESCRIPTION: compile time configurable options for 
 *								 : XNotesPlus
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log: config.h,v $
 * Revision 1.1.1.1  2004/10/11 17:35:05  mjhammel
 * Initial import
 *
 * Revision 1.6  2004/02/20 04:49:23  mjhammel
 * Added CUPS support for printing.
 * Fixed monthly display in appt's to only highlight date for current month.
 * Changed notebook layout in appt's.
 *
 * Revision 1.5  2004/01/04 18:13:12  mjhammel
 * Migrated pilot-tools to make use of user installed pilot-link package
 * instead of locally included package in order to support USB devices.
 * Changed HideNote() to save note before hiding and XPSaveNote() to not
 * save hidden notes.  This prevents hidden notes from getting their sizes
 * munged since they don't have a visible window.
 * Removed extra fclose() in xn-todos.c that caused segfaults.
 *
 * Revision 1.4  2002/04/25 03:52:58  mjhammel
 * Updates to appointment interface.
 *
 * Revision 1.3  2002/02/15 15:47:55  mjhammel
 * Added NOPILOT config option; updated website
 *
 * Revision 1.2  2002/01/27 21:38:14  mjhammel
 * Builds under XFree86 4.1 *and* Red Hat 7.1 were being done as parallel
 * compiles.  This broke the XNotesPlus build environment.  These fixes force
 * the compiles to be sequential.
 *
 * Also fixed the fonts in the XNotesPlus.rc.themed file for use under XFree86
 * 4.1.  They were broke anyway, but just happened to work previously.
 *
 * Revision 1.1.1.1  2002/01/10 18:37:48  mjhammel
 * Initial checkin
 *
 *
 *========================================================================*/

/*
 * Where are the libraries and include files? 
 * 
 * Uncomment these for use with GTK.  The "gtk-config" script will automatically
 * determine the location of headers and libraries.  If this script is not
 * in your PATH then you can specify the full path to it here.
 * 
GTK_LIB=`gtk-config --libs`
GTK_INC=`gtk-config --cflags`
 * 
 */
GTK_LIB=`gtk-config --libs`
GTK_INC=`gtk-config --cflags`

/*
 * use this option if you want to use CUPS based printing.  Without it,
 * XNotesPlus will use the printcmd option in the configuration file.
 */
#define USE_CUPS

/*
 * Uncomment this one if you don't want to use the Pilot tools but still want
 * the Address Manager, Todo List, etc.  You'd use this, for example, if you
 * don't have a Pilot to sync with.
#define NOPILOT
 */

/*
 * These two are required if you *ARE* using the Pilot tools.  
 * PILOTINC must be the directory where your header files (such as
 * pi-dlp.h) are installed.
 * PILOTLIB must be the fully qualified path to the libpisock.a library.
 */
PILOTINC=/usr/include
PILOTLIB=/usr/lib/libpisock.a

/*
 * Uncomment the next line if you want to build Nenscript.  Nenscript is
 * used by the Address Manager to print envelopes.  It may not build on all
 * systems but seems fine on Red Hat Linux.  I didn't write it - I'm just 
 * including it for completeness sake.
 */
#define DO_NENSCRIPT

/*
 * Uncomment this is you use Linux or FreeBSD and CDE and get unresolved
 * references to "_Xsetlocal".
 *
#define LINUX_CDE
 */

/*
 * If the above still doesn't work for you, try this one instead.  Only 
 * this one *or* the previous one should be uncommented, not both.
 */
#define NO_XSETLOCALE

/*
 * If you use the GTK libraries compiled with X Input enabled then
 * uncomment the following line.  Neither Motif nor LessTif supports this 
 * option.  It doesn't hurt to define this with Motif and LessTif, but it does
 * if your GTK does *not* support USE_XINPUT.
 */
#define USE_XINPUT

/*
 * Uncomment this next one if you want to use the fancy background pixmaps
 * in the menus and other places.  Its nice if you have high color
 * displays, but don't bother if you have an 8 bit (256 color display).
 *
 * BTW, this option will make use of the XNotesPlus.{ad,rc}.themed application
 * defaults files.  If you want the old version, which uses
 * XNotesPlus.{ad,rc}.standard, then don't define this.
 * 
 * Note:  This can be overwritten by using the graphical installer in v3.5.
 * 
 */
#define USE_FANCY_XPM

/*
 * Need a statically linked binary?  Uncomment the following.
 * If you're on Linux (or some other system that uses dlopen()) then you
 * need to uncomment USE_DLLIB too.
 * 
 * NOTE:  this may not work properly - the gdk_imlib routines seem to get
 * confused by static linking.
 *
#define MAKE_STATIC
#define USE_DLLIB
 */

/*
 * Leave this uncommented if you want to use spell checking using an 
 * ispell compatible spell checker (such as aspell).
 */
#define USE_PSPELL

/*
 * Uncomment the following if you use a system that doesn't know about
 * bzero() - like Solaris, for example.
 */
#define NO_BZERO

/*
 * Uncomment this if you want to turn the debug macros off.  By default
 * the macros expand into real source code.  Note that even if
 * expanded, no debug output will be generated unless you use the appropriate
 * command line options: --debugfile <file> and --debuglevel <level>
 *
 * This isn't really very useful unless you want to seriously reduce the
 * binary size.
#define DEBUGOFF
 */

/*
 * Uncomment this if you use gcc.  If your "cc" is a sym-link to gcc or
 * you've renamed gcc to cc, then uncomment this as well.  You don't
 * have to (I don't think), but it helps.
 */
#define HASGCC

/*
 * Define the tar command use for building a distributable package here.
 * Almost no one (except people like me who use two different versions of
 * tar) will need to change this.
TAR=gtar cvzf 
 */
TAR=tar cvzf 

/*=========================================================================
 * End of configurable options.
 * You don't need to change anything below this line.
 *=========================================================================*/

/*
 * Future expansion:
 * Uncomment this is you want to build the graphical installer.
 */
#define DO_INSTALLGUI
APPTS=-DUSE_APPT

/*
 * Pspell configuration.
 */
#ifdef USE_PSPELL
PSPELLDEF = -DDO_PSPELL
#else
PSPELLDEF = 
#endif

/*
 * Pilot configuration.
 */
#ifdef NOPILOT
NOPILOTDEF = -DNO_PILOT
#else
NOPILOTDEF = 
#endif

/*
 * CUPS configuration
 */
#ifdef USE_CUPS
USECUPS = -DUSE_CUPS
CUPSLIB = -lcups
#else
USECUPS = 
CUPSLIB = 
#endif

/*
 * Do we need to hack around the bzero() problem?
 */
#ifdef NO_BZERO
USE_MEMSET = -DSYSV_USE_MEMSET
#endif

#ifdef NO_XSETLOCALE
LOCAL_XSETLOCALE = -DUSE_LOCAL_XSETLOCALE
#endif

/*
 * Put all those defines together.
 */
OTHER_DEFINES = $(USE_MEMSET) $(LOCAL_XSETLOCALE) $(PSPELLDEF) $(NOPILOTDEF) $(APPTS) $(USECUPS)


/*
 * Compile and Link options; complex, but thats what we get when we don't
 * compile with the same compiler across platforms.
 * DEGBUG - turns on local debugging library
 * -Wall  - turns on exhaustive checking by gcc compiler
 * -Wno-unused  - turns off notices about unused parameters and variables.
 *		This is useful because the GUI has lots of unused parameters
 *		in callbacks and these are required by only some compilers.
 */
#ifdef DEBUGOFF
#    ifdef HASGCC
CC = gcc
DEFINES   = -g -Wall $(OTHER_DEFINES)
#    else
DEFINES   = $(OTHER_DEFINES)
#    endif
#else
#    ifdef HASGCC
CC = gcc
DEFINES   = -g -Wall -DDEBUG $(OTHER_DEFINES)

/* Override parallel builds - they break the build environment. */
PARALLELMFLAGS = -j 1

#    else
DEFINES   = -DDEBUG $(OTHER_DEFINES)
#    endif
#endif

/*
 * Set up for which version to build.
 */
#ifdef USE_FANCY_XPM
DEFINE_USE_FANCY_XPM = -DUSE_FANCY_XPM
#endif

DEFINE_USE_GTK12 = -DUSE_GTK12
EXTRA_DEFINES = -DGTK_DISABLE_COMPAT_H -DDO_GTK $(DEFINE_USE_GTK12) $(DEFINE_USE_FANCY_XPM)

/*
 * Add X Input library, if necessary.
 */
#ifdef USE_XINPUT
XINPUT = -lXi
#else
XINPUT = 
#endif

/*
 * For the masochistic types, who need a static binary.
 */
#ifdef MAKE_STATIC
EXTRA_LDOPTIONS = -static
#endif
#ifdef USE_DLLIB
DLLIBS = -ldl
#endif
