/* $Id: prototypes.h,v 1.4 2005/01/22 15:20:56 mjhammel Exp $ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2001
 *========================================================================
 *              FILE NAME: prototypes.h
 *            DESCRIPTION: External function definitions
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log: prototypes.h,v $
 * Revision 1.4  2005/01/22 15:20:56  mjhammel
 * Removed registration dialog and all references to it.
 *
 * Revision 1.3  2004/11/15 23:53:13  mjhammel
 * First working version of category editor.
 *
 * Revision 1.2  2004/11/15 20:28:55  mjhammel
 * Added support for menus in generic shells (generic.c).
 * Changed to menus from button bar in Todo Manager.
 * Added first pass at Category Edit dialog.
 *
 * Revision 1.1.1.1  2004/10/11 17:35:06  mjhammel
 * Initial import
 *
 * Revision 1.8  2004/03/13 15:11:28  mjhammel
 * Added timers to check if day changed to update hilite in Monthly view of
 * Calendar Manager.
 * Changed version to 3.6.1.
 * Updated History for 3 changes (so far) for 3.6.1.
 *
 * Revision 1.7  2004/02/26 05:22:50  mjhammel
 * Changed about to use gnome-moz-remote.
 * Fixed crash bug in findnote.c
 * Cleaned out all references to ical.
 * Added default browser config as a backup if gnome-moz-remote isn't found.
 * Fixed packaging so xn-cal is included.
 * Changed "Calendar Tool" to "Calendar Manager"
 * Removed signal(SIGCHLD, SIG_IGN) because gdb says we're calling wait() too.
 * Moved check for registration outside of config file parsing.
 *
 * Revision 1.6  2004/02/20 04:49:23  mjhammel
 * Added CUPS support for printing.
 * Fixed monthly display in appt's to only highlight date for current month.
 * Changed notebook layout in appt's.
 *
 * Revision 1.5  2004/02/17 14:25:54  mjhammel
 * Added first pass at alarm handling for appointments.
 *
 * Revision 1.4  2004/01/23 04:22:41  mjhammel
 * Added WriteAppt and FreeAppt to c_pilot.c
 * Added appointment related defines to XNotesplus.h
 * Added appointment prototypes to prototypes.h
 * Added PopulateCalendar() stub to appointments.c
 * Fixed bug in todos that caused crash when todo entry was deleted.
 *
 * Revision 1.3  2002/04/05 00:42:40  mjhammel
 * Added Cancel button to Pilot HotSync dialog (plus some other minor updates).
 *
 * Revision 1.2  2002/02/18 21:49:15  mjhammel
 * First pass at online help
 *
 * Revision 1.1.1.1  2002/01/10 18:37:59  mjhammel
 * Initial checkin
 *
 *
 *========================================================================*/
#ifndef XP_PROTOS_H
#define XP_PROTOS_H

#ifndef XP_NOTES_C
extern void XPCreateNewNote();
extern void XPRaiseNotes();
extern void XPRaiseAllNotes();
extern void XPLowerNotes();
extern void XPLowerAllNotes();
extern void XPSaveAllNotes();
extern void XPUnHideAllNotes();
extern void XPHideAllNotes();
extern void XPCascadeNotes();
extern void XPMakeNoteListCB();
extern void	XPMakeNoteWidget();
extern void XPAnchorNote();
extern void XPUnAnchorNote();
extern void	XPDestroyNote();
extern void	XPSetSaveSensitive();
extern void	XPTearOff();
extern void XPHideNote();
extern void XPEraseNote();
extern void XPSaveNote();
extern void XPGetNotePosition();
extern StickyNote	*XPFindNote();
extern StickyNote	*XPAllocNote();
#endif /* XP_NOTES_C */


#ifndef XP_BINDINGS_C
extern void XPActionAlarm();
extern void XPActionAnchor();
extern void XPActionDestroy();
extern void XPActionHide();
extern void XPActionErase();
extern void XPActionCal();
extern void XPActionSave();
extern void XPActionPrint();
extern void XPActionOpen();
extern void XPActionEnvelope();
extern void XPActionProject();
extern void XPActionFind();
extern void XPActionName();
extern void XPActionDate();
extern void XPActionEmail();
extern void XPActionExport();
extern void XPActionAddressManager();
extern void XPSetNoteName();
extern void XPSendNote();
#endif /* XP_BINDINGS_C */


#ifndef XP_MENUS_C
extern void XPCreateMainMenu();
extern int XPButtonPress();
#endif /* XP_MENUS_C */


#ifndef XP_UTILS_C
extern void			*XPInsertText();
extern void			XPShutdown();
extern void			XPCenterWindow();
extern int			XPLaunchIcal();

extern GtkWidget	*XPMultiLineText();
extern GtkWidget	*XPGtkNameToWidget();
extern int			XPNoop2();
extern void			XPGTKShowDisabledCmds();
extern int			XPFind();
extern int			XPUpdateInsertPtr();

#endif /* XP_UTILS_C */


#ifndef XP_PLAID_C
extern GtkWidget	*XPCreatePlaidWidget();
#endif /* XP_PLAID_C */


#ifndef XP_TODOS_S
extern void	XPTodos();
#endif /* XP_TODO_S */


#ifndef XP_LOADSAVE_C
extern void	XPLoadSavedNotes();
#endif /* XP_LOADSAVE_C */

#ifndef XP_TT_C
extern void	XPToolTips();
#endif /* XP_TT_C */

#ifndef XP_FINDTEXT_C
extern void	XPFindTextPrompt();
#endif /* XP_FINDTEXT_C */

#ifndef XP_PROJECTS_C
extern void	XPProject();
extern void	XPNoteProject();
extern void	XPUpdateColorBox();
extern void	XPUpdateProject();
#endif /* XP_PROJECTS_C */

#ifndef XP_TIMERCB_C
extern void	XPAutoSave();
extern void	XPAlarmCheck();
extern void	XPCreateTimerPrompt();
extern void XPAlarmOff();
#endif /* XP_TIMERCB_C */


#ifndef XP_ERRDIALOG_C
extern void XPErrPopDown();
#endif /* XP_ERRDIALOG_C */


#ifndef XP_FINDNOTE_C
extern void	XPCreateFindNotePrompt();
extern void	XPGenericNoteList();
#endif /* XP_FINDNOTE_C */

#ifndef XP_CONFIRM_C
extern void	XPConfirmIt();
extern void	XPConfirmDialogClose();
#endif /* XP_CONFIRM_C */


#ifndef XP_COLORS_C
extern int	XPSetGlobalColors();
#endif /* XP_COLORS_C */


#ifndef XP_IO_C
extern void	XPOpenNote();
extern void	XPExportNote();
#endif /* XP_IO_C */


#ifndef XP_PRINT_C
extern void	XPPrintNote();
extern int XPPrintFile();
#endif /* XP_PRINT_C */


#ifndef XP_MERGE_C
extern void	XPMergeNotes();
#endif /* XP_MERGE_C */


#ifndef XP_NAME_C
extern void	XPNameNote();
extern void	XPUpdateName();
#endif /* XP_NAME_C */


#ifndef XP_INSERTS_C
extern void	XPInsertDates();
extern void	XPInsertCalendar();
#endif /* XP_INSERTS_C */


#ifndef XP_EMAIL_C
extern void	XPCreateEmailNotePrompt();
extern void	XPEmailNote();
extern void	XPFillEmailList();
#endif /* XP_EMAIL_C */


#ifndef XP_CONFIG_C
extern void	XPReadConfigFile();
#endif /* XP_CONFIG_C */


#ifndef XP_AUDIO_C
extern void	XPSetUpAudio();
#endif /* XP_AUDIO_C */


#ifndef XP_ENVELOPE_C
extern void	XPEnvelope();
extern void	XPUpdateToText();
#endif /* XP_ENVELOPE_C */

#ifndef XP_DIALOG_C
extern void	XPMsgWindow();
extern void	XPCloseMsgWindow();
#endif /* XP_DIALOG_C */

#ifndef XP_ABOUT_C
extern int XPAbout();
#endif /* XP_ABOUT_C */

#ifndef XP_CALENDAR_C
extern void XPShowCalendar();
extern void XPCalendarUpdate();
extern void XPCalendarGetDate();
extern void XPCloseCalendar();
#endif /* XP_CALENDAR_C */

#ifndef XP_PILOT_C
extern void XPPilot();
#endif /* XP_PILOT_C */

#ifndef XP_PADDR_C
#endif /* XP_PADDR_C */

#ifndef XP_ADDR_MGR_C
extern int XPAddressManager();
extern int XPUpdateAddrList();
#endif /* XP_ADDR_MGR_C */

#ifndef XP_ADDR_EDITOR_C
extern int XPAddressEditor();
extern int XPCloseAddrEditor();
#endif /* XP_ADDR_EDITOR_C */

#ifndef XP_SPELLCHECK_C
extern GtkWidget	*XPSpellCheckPrompt();
#endif /* XP_SPELLCHECK_C */

#ifndef XP_GENERIC_C
extern GtkWidget	*XPCreateGenericShell();
#endif /* XP_GENERIC_C */

#ifndef XP_CATEDIT_C
extern void	XPCatEdit();
#endif /* XP_CATEDIT_C */

#ifndef XP_APPT_C
extern GtkWidget	*XPAppointments();
extern GtkWidget	*XPApptAlarmCheck();
extern GtkWidget	*XPApptInitDayCheck();
#endif /* XP_APPT_C */

/*
 * Common files (in libxncommon)
 */
#ifndef XP_CUTILS_S
extern char			*XPSafeAlloc();
extern void			*XPSetNoteDir();
extern char			*XPMakeFname();
extern void 		XPPopUpList();
extern void			XPNoop();
extern void			XPRingBell();
extern void			XPErrPopUp();
extern void			XPShowDisabledCmds();
extern int			XPTestCmd();
extern void			XPFixUpPath();
extern void			XPConvertCase();
extern int			XPFindString();
extern void			XPHelp();
#endif /* XP_CUTILS_S */

#ifndef XP_C_C
extern void			XPValidateAppRes();
extern void			XPShowDefaults();
#endif /* XP_C_C */

#ifndef XP_CPROJECTS_S
extern void	XPSetUpProjects();
extern void	XPWriteProjects();
#endif /* XP_CPROJECTS_S */

#ifndef XP_CPADDR_C
extern void XPPilotParseDBList();
extern void XPFreeMemoList();
extern int XPPilotParseMemos();
extern void XPWriteMemos();
extern int XPPilotParseAddr();
extern void XPWriteAddresses();
extern void XPAddAddress();
extern void XPFreeCategoryList();
extern void XPFreeAddressList();
extern void XPGenerateMemoCategoryList();
extern void XPWriteTodos();
extern int XPPilotParseTodo();
extern void XPPilotWriteAppointment();
extern int XPPilotParseAppointment();
extern void XPFreeApptList();
extern void XPGenerateCategoryList();
#endif /* XP_CPADDR_C */

#endif /* XP_PROTOS_H */
