/* $Id: about.c,v 1.2 2005/01/22 15:20:56 mjhammel Exp $ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2001
 *========================================================================
 *              FILE NAME: about.c 
 *            DESCRIPTION: gratuitous self-promotion
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPAbout
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log: about.c,v $
 * Revision 1.2  2005/01/22 15:20:56  mjhammel
 * Removed registration dialog and all references to it.
 *
 * Revision 1.1.1.1  2004/10/11 17:35:32  mjhammel
 * Initial import
 *
 * Revision 1.2  2004/02/26 05:22:50  mjhammel
 * Changed about to use gnome-moz-remote.
 * Fixed crash bug in findnote.c
 * Cleaned out all references to ical.
 * Added default browser config as a backup if gnome-moz-remote isn't found.
 * Fixed packaging so xn-cal is included.
 * Changed "Calendar Tool" to "Calendar Manager"
 * Removed signal(SIGCHLD, SIG_IGN) because gdb says we're calling wait() too.
 * Moved check for registration outside of config file parsing.
 *
 * Revision 1.1.1.1  2002/01/10 18:38:04  mjhammel
 * Initial checkin
 *
 *
 *========================================================================*/
#define XP_ABOUT_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void		ClearAbout();


/* === Global Variables === */
extern GtkWidget		*toplevel;
extern AppRes			app_res;
extern ExternalCmds	external_cmds;

/* === Static Variables === */
static GtkWidget		*aboutwidget;


/*========================================================================
 *	Name:			XPAbout
 *	Prototype:	XPAbout()
 *					
 *	Description:
 * Put up a window that says who I am and what this thing is.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPAbout(
)
{
#ifdef DEBUG
	char	fname[]="XPAbout";
#endif

	GtkWidget		*vbox, *newvbox;
	GtkWidget		*logo, *logo_pixmap;
	GtkWidget		*cancel_button;
	GtkWidget		*title, *info;
	GtkWidget		*eventbox;
	GtkStyle			*style;
	static GdkPixmap		*gm_logo_xbm=NULL;
	static GdkPixmap		*details_xbm=NULL;
	static GdkBitmap		*details_mask;
	static GdkBitmap		*gm_logo_mask;
	char				buf[256];

	char				*msg=XP_S_PROG_NAME;
	char				*msg2;

	GShell_T			gshell;

	static char		firsttime = TRUE;

	/*
	 * Allow only one copy at a time.
	 */
	if ( aboutwidget != NULL )
		return;

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	DBGPrintf(DBG_INFO,("Creating find dialog.\n"));
	aboutwidget = XPCreateGenericShell(AboutDialog, 
						CANCEL_BUTTON, &gshell, TRUE );

	gtk_signal_connect (GTK_OBJECT (aboutwidget), "destroy",
			GTK_SIGNAL_FUNC (ClearAbout), NULL);

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	vbox = gshell.vbox;
	cancel_button = gshell.cancel;

	/*
	 * A set of labels for descriptive info and pixmaps
	 */

	/*
	 * Labels don't have backgrounds (their rendered transparent over their 
	 * parent windows).  So we stuff an eventbox in here to add the
	 * background pixmap.
	 */
	eventbox = gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(vbox), eventbox);
	gtk_widget_show (eventbox);

	newvbox = gtk_vbox_new(FALSE, 1);
	gtk_container_add(GTK_CONTAINER(eventbox), newvbox);
	gtk_widget_show(newvbox);

	logo = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(logo), GTK_SHADOW_NONE);
	gtk_widget_show(logo);
	gtk_container_add(GTK_CONTAINER(newvbox), logo);

	title = gtk_label_new(msg);
	gtk_widget_set_name(title, "title");
	gtk_widget_show(title);
	gtk_container_add(GTK_CONTAINER(newvbox), title);

	style = gtk_style_copy( gtk_widget_get_style( eventbox ) );

	if ( firsttime )
	{
		sprintf(buf, "%s/%s", app_res.xpm_dir, XP_BG_1_XPM);
		details_xbm = gdk_pixmap_create_from_xpm(
				toplevel->window,
				&details_mask,
				&style->bg[GTK_STATE_NORMAL],
				buf);

		sprintf(buf, "%s/%s", app_res.xpm_dir, XP_GM_LOGO_XPM);
		gm_logo_xbm = gdk_pixmap_create_from_xpm(
				toplevel->window,
				&gm_logo_mask,
				&style->bg[GTK_STATE_NORMAL],
				buf);

		firsttime = FALSE;
	}
	style->bg_pixmap[GTK_STATE_NORMAL] = details_xbm;
	gtk_widget_set_style(eventbox, style);
	gdk_pixmap_ref(details_xbm);
	gdk_pixmap_ref(gm_logo_xbm);

	logo_pixmap = gtk_pixmap_new(gm_logo_xbm, gm_logo_mask);
	gtk_container_add(GTK_CONTAINER(logo), logo_pixmap);
	gtk_widget_show( logo_pixmap );

	msg2 = (char *)XPSafeAlloc(strlen(XP_S_ABOUT_INFO) + strlen(VERSION) + 1);
	strcpy(msg2, VERSION);
	strcat(msg2, XP_S_ABOUT_INFO);
   info = gtk_label_new(msg2);
	gtk_widget_show(info);
	gtk_widget_set_name(info, "about");
	gtk_container_add(GTK_CONTAINER(newvbox), info);
	free(msg2);

	/*
	 * Cancellation button.
	 */
	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
			GTK_SIGNAL_FUNC (ClearAbout), NULL);

	/*
	 * Prevent resizing.
	 */
	gtk_window_set_policy(GTK_WINDOW(aboutwidget), FALSE, FALSE, FALSE);

	/*
	 * Stick the dialog in the middle of the display.
	 */
	gtk_widget_realize( aboutwidget );
	XPCenterWindow(aboutwidget, NULL, aboutwidget);
	gtk_widget_show(aboutwidget);

}

/*
 * ClearAbout - get rid of the confirmation box.
 */
static void
ClearAbout()
{
	if ( aboutwidget != NULL )
	{
		gtk_widget_destroy( aboutwidget );
		aboutwidget = NULL;
	}
}
