/* $Id: generic.c,v 1.3 2004/11/15 23:53:13 mjhammel Exp $ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2001
 *========================================================================
 *              FILE NAME: generic.c
 *            DESCRIPTION: generic shell window for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPCreateGenericShell
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log: generic.c,v $
 * Revision 1.3  2004/11/15 23:53:13  mjhammel
 * First working version of category editor.
 *
 * Revision 1.2  2004/11/15 20:28:55  mjhammel
 * Added support for menus in generic shells (generic.c).
 * Changed to menus from button bar in Todo Manager.
 * Added first pass at Category Edit dialog.
 *
 * Revision 1.1.1.1  2004/10/11 17:35:32  mjhammel
 * Initial import
 *
 * Revision 1.1.1.1  2002/01/10 18:38:05  mjhammel
 * Initial checkin
 *
 *
 *========================================================================*/
#define XP_GENERIC_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>

#include <gtk/gtk.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void CreateMenuBar();
static void AddMenuItem();


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;


/* === Static Variables === */



/*========================================================================
 *	Name:			XPCreateGenericShell
 *	Prototype:	XPCreateGenericShell( char *, unsigned char )
 *
 *	Description:
 *		This is the basic shell in which most of the other pop up 
 *		windows will be placed.  This gets called each time a new
 *		window needs to be popped up.
 *
 *	Input Arguments:
 *		char 			*textstring			The title to use for the window
 *		u_char 		button_types		Defines which buttons to include
 *												Valid values are:
 *												0x0001		Accept button
 *												0x0002		Cancel button
 *												0x0004		OK button
 *												0x0008		Overwrite button
 *												0x0010		Append button
 *												0x0020		Help button
 *												0x0040		Print button
 *												0x0080		Yes button
 *												0x0100		Close button
 *												0x0200		No button
 *												0x0400		Search button
 *												0x0fff		All buttons (never used)
 *												0x8000		Create menu bar
 *	Output Arguments:
 *		None.
 *	Return Values:
 *		GtkWidget	*widget_id			The widget id of the shell which will be the
 *												parent of the rest of the windows widgets.
 *
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *		In order to use this routine, the caller will need to know the names
 *		of the child widgets of the parent widget id which is returned.
 *		These will be used to set callbacks for the buttons and to find the form
 *		widget in which all the other widgets specific to the caller will go.
 *		The caller can use the Xt function XtNameToWidget() to get the widget
 *		id of the widget for which a callback will be set or for the form widget.
 *
 *		All of these names of these widgets are suffixed by the widget id
 *		of the shell widget (ie the id returned by this routine).
 *		These are the names the caller needs to know:
 *
 *		Widget Name				Purpose
 *		-----------				------------------------------------
 *		GenericForm				the form in which the caller will place
 *									the widgets specific to that caller.
 *		GenericHelpB			The Help button for this shell.
 *		GenericAcceptB			The Accept button for this shell.
 *		GenericCancelB			The Cancel button for this shell.
 *		GenericOverwriteB		The Overwrite button for this shell.
 *		GenericAppendB			The Append button for this shell.
 *
 *		Remember, tack on the widget id of the shell!  For example, if the
 *		shell widget id is 138457 (decimal) then the name of the form widget
 *		for this shell is "GenericForm138457".
 *		
 *========================================================================*/

GtkWidget *
XPCreateGenericShell(
	char				*textstring,
	guint32			button_types,
	GShell_T			*caller_data,
	gboolean			usebg
)
{

#ifdef DEBUG
	char 			fname[]="XPCreateGenericShell()";
#endif

	char			buf[256];
	char			shellname[256];
	GtkWidget	*shell_id, *inside_form, *callers_form, *callers_frame;
	GtkWidget	*vbox2, *eventbox;
	GtkWidget	*callers_eventbox;
	GtkWidget	*button_frame, *button_form, 
					*button_accept, *button_cancel, *button_overwrite,
					*button_print, *button_search, 
					*button_help, *button_append, *label;


	DBGEnter();

	/*
	 * Create the pop up shell
	 */

	/* new version of title bar */
	(void) sprintf ( buf, "%s - %s", XP_S_PROG_NAME, textstring );
	sprintf(shellname, "GenericShell");

	/* The Shell gets its own window. */
	shell_id = gtk_window_new(GTK_WINDOW_DIALOG);
	if ( ( button_types & XP_GENERIC_MENU ) == XP_GENERIC_MENU )
		gtk_container_set_border_width (GTK_CONTAINER (shell_id), 0);
	else
		gtk_container_set_border_width (GTK_CONTAINER (shell_id), 5);
	gtk_window_set_title(GTK_WINDOW(shell_id), buf);
	gtk_widget_set_name(shell_id, shellname); 

	/* Save the shell id for the caller. */
	caller_data->shell_id = shell_id;

	/* All Generic shell's have the same WM class id */
	gtk_window_set_wmclass(
		GTK_WINDOW(shell_id),
		"StickyNote",
		StickyNoteClass);
	
	/* This should get overridden by the caller. */
	gtk_signal_connect (GTK_OBJECT (shell_id), "delete_event",
			GTK_SIGNAL_FUNC (gtk_false), NULL);

	/* create the inside form - this one is hidden from the caller */
	inside_form = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(inside_form);
	gtk_container_add(GTK_CONTAINER(shell_id), inside_form);

	/*
	 * create the two frames which define the callers area and the buttons
	 * area.
	 */

	/* create the callers frame */
	callers_eventbox = eventbox = gtk_event_box_new();
	if ( usebg )
		gtk_widget_set_name(eventbox, shellname);
	gtk_widget_show(eventbox);

	vbox2 = gtk_vbox_new(FALSE, 2);
	gtk_widget_show(vbox2);
	gtk_container_add(GTK_CONTAINER(eventbox), vbox2);
 
	callers_frame = gtk_frame_new("");
	gtk_frame_set_label(GTK_FRAME(callers_frame), NULL);
	gtk_widget_show(callers_frame);
	gtk_box_pack_start(GTK_BOX(vbox2), callers_frame, FALSE, FALSE, 0);

	eventbox = gtk_event_box_new();
	gtk_widget_set_name(eventbox, shellname);
	gtk_widget_show(eventbox);
	gtk_container_add(GTK_CONTAINER(callers_frame), eventbox);

	callers_form = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(callers_form);
	gtk_container_add(GTK_CONTAINER(eventbox), callers_form);
	gtk_container_set_border_width (GTK_CONTAINER (callers_form), 5);

	/* Save this widget id for the caller. */
	caller_data->vbox = callers_form;


	/*
	 * What we do from here depends on if we're making a menu bar or a
	 * button bar for this generic shell.
	 */
	if ( ( button_types & 0x8000 ) == XP_GENERIC_MENU )
	{
		/* Go make the menu bar */
		CreateMenuBar(inside_form, caller_data);

		/* Add the users area after the menu */
		gtk_container_add(GTK_CONTAINER(inside_form), callers_eventbox);

		/* return to the caller */
		DBGExit();
		return (shell_id);
	}
	

	/* Add the callers eventbox here, in case menus were going in front of it */
	gtk_container_add(GTK_CONTAINER(inside_form), callers_eventbox);

	/* create the buttons frame */
	button_frame = gtk_frame_new("");
	gtk_frame_set_label(GTK_FRAME(button_frame), NULL);
	gtk_widget_show(button_frame);
	gtk_box_pack_start(GTK_BOX(inside_form), button_frame, FALSE, FALSE, 0);

	eventbox = gtk_event_box_new();
	gtk_widget_set_name(eventbox, "menu_bar");
	gtk_widget_show(eventbox);
	gtk_container_add(GTK_CONTAINER(button_frame), eventbox);

	button_form = gtk_hbox_new(FALSE, 0);
	gtk_widget_set_name(callers_form, "menu_bar");
	gtk_widget_show(button_form);
	gtk_container_add(GTK_CONTAINER(eventbox), button_form);


	/* create the buttons in the button area */

	if (	( button_types & ACCEPT_BUTTON ) || ( button_types & OK_BUTTON ) || 
			( button_types & YES_BUTTON ) )
	{
		sprintf(buf, "GenericAcceptB%d", (int)shell_id);
		if ( button_types & ACCEPT_BUTTON )
			sprintf(buf, "Accept");
		else
		if ( button_types & OK_BUTTON )
			sprintf(buf, "OK");
		else
			sprintf(buf, "Yes");

		/*
		 * GTK doesn't allow resource settings as easily as with
		 * Xt, so we have to work around it by manually specifing
		 * the label in the button.
		 */
		button_accept = gtk_button_new();
		gtk_widget_set_name(button_accept, "window_button");
		label = gtk_label_new(buf);
		gtk_widget_set_name(label, "window_button");
		gtk_widget_show(label);
		gtk_container_add(GTK_CONTAINER(button_accept), label);
		gtk_box_pack_start (GTK_BOX (button_form), button_accept, 
			FALSE, FALSE, 2);
		gtk_widget_show(button_accept);

		caller_data->accept = button_accept;
	}
	
	if (	( button_types & CANCEL_BUTTON ) || ( button_types & CLOSE_BUTTON ) ||
			( button_types & NO_BUTTON ) )
	{
		sprintf(buf, "GenericCancelB%d", (int)shell_id);
		if ( button_types & CANCEL_BUTTON )
			sprintf(buf, "Cancel");
		else
		if ( button_types & CLOSE_BUTTON )
			sprintf(buf, "Close");
		else
			sprintf(buf, "No");

		button_cancel = gtk_button_new();
		gtk_widget_set_name(button_cancel, "window_button");
		label = gtk_label_new(buf);
		gtk_widget_set_name(label, "window_button");
		gtk_widget_show(label);
		gtk_container_add(GTK_CONTAINER(button_cancel), label);
		gtk_box_pack_start (GTK_BOX (button_form), button_cancel, 
			FALSE, FALSE, 2);
		gtk_widget_show(button_cancel);

		caller_data->cancel = button_cancel;
	}

	if ( button_types & PRINT_BUTTON )
	{
		sprintf(buf, "Print");

		/*
		 * GTK doesn't allow resource settings as easily as with
		 * Xt, so we have to work around it by manually specifing
		 * the label in the button.
		 */
		button_print = gtk_button_new();
		gtk_widget_set_name(button_print, "window_button");
		label = gtk_label_new(buf);
		gtk_widget_set_name(label, "window_button");
		gtk_widget_show(label);
		gtk_container_add(GTK_CONTAINER(button_print), label);
		gtk_box_pack_start (GTK_BOX (button_form), button_print, 
			FALSE, FALSE, 2);
		gtk_widget_show(button_print);

		caller_data->print = button_print;
	}
	
	if ( button_types & SEARCH_BUTTON ) 
	{
		sprintf(buf, "Search");

		/*
		 * GTK doesn't allow resource settings as easily as with
		 * Xt, so we have to work around it by manually specifing
		 * the label in the button.
		 */
		button_search = gtk_button_new();
		gtk_widget_set_name(button_search, "window_button");
		label = gtk_label_new(buf);
		gtk_widget_set_name(label, "window_button");
		gtk_widget_show(label);
		gtk_container_add(GTK_CONTAINER(button_search), label);
		gtk_box_pack_start (GTK_BOX (button_form), button_search, 
			FALSE, FALSE, 2);
		gtk_widget_show(button_search);

		caller_data->search = button_search;
	}
	
	if ( button_types & OVERWRITE_BUTTON )
	{
		sprintf(buf, "GenericOverwriteB%d", (int)shell_id);
		sprintf(buf, "Overwrite");
		button_overwrite = gtk_button_new();
		gtk_widget_set_name(button_overwrite, "window_button");
		label = gtk_label_new(buf);
		gtk_widget_set_name(label, "window_button");
		gtk_widget_show(label);
		gtk_container_add(GTK_CONTAINER(button_overwrite), label);
		gtk_box_pack_start (GTK_BOX (button_form), button_overwrite, 
			FALSE, FALSE, 2);
		gtk_widget_show(button_overwrite);

		caller_data->overwrite = button_overwrite;
	}

	if ( button_types & APPEND_BUTTON )
	{
		sprintf(buf, "GenericAppendB%d", (int)shell_id);
		sprintf(buf, "Append");

		button_append = gtk_button_new();
		gtk_widget_set_name(button_append, "window_button");
		label = gtk_label_new(buf);
		gtk_widget_set_name(label, "window_button");
		gtk_widget_show(label);
		gtk_container_add(GTK_CONTAINER(button_append), label);
		gtk_box_pack_start (GTK_BOX (button_form), button_append, 
			FALSE, FALSE, 2);
		gtk_widget_show(button_append);

		caller_data->append = button_append;
	}

	if ( button_types & HELP_BUTTON )
	{
		sprintf(buf, "GenericHelpB%d", (int)shell_id);
		sprintf(buf, "Help");

		button_help = gtk_button_new();
		gtk_widget_set_name(button_help, "window_button");
		label = gtk_label_new(buf);
		gtk_widget_set_name(label, "window_button");
		gtk_widget_show(label);
		gtk_container_add(GTK_CONTAINER(button_help), label);
		gtk_box_pack_start (GTK_BOX (button_form), button_help, 
			FALSE, FALSE, 2);
		gtk_widget_show(button_help);

		caller_data->help = button_help;
	}


	DBGExit();

	/* return to the caller */
	return (shell_id);

}

/*========================================================================
 *	Name:			CreateMenuBar
 *	Prototype:	CreateMenuBar( GtkWidget *, GtkWidget *, GShell_T *)
 *
 * Input Arguments:
 * GtkWidget	*shell_id		Main window of this dialog.
 * GtkWidget	*inside_form	Vbox inside which everything else is placed.
 * GShell_T		*caller_data	Holds menu information from caller.
 *		
 *========================================================================*/
static void
CreateMenuBar(
	GtkWidget	*inside_form,
	GShell_T		*caller_data
)
{
#ifdef DEBUG
	char 			fname[]="CreateMenuBar()";
#endif

	GtkWidget	*button;
	GtkWidget	*menubar;
	GMenus_T		menus;
	int			found = 0;
	int			i;

	DBGEnter();
	static char	*Menu_Labels[] = {
			XP_S_GFILE, XP_S_GEDIT, XP_S_GOPTIONS, XP_S_GHELP, NULL
			};
	
	/* Clear the menu structure. */
	bzero((char *)&menus, sizeof(GMenus_T));

	/* Verify there is a link list of menu items to grab */
	if ( caller_data->linkhdr == NULL ) 
		return;

	/* Run through the link list and add menu items to the menus. */
	g_list_foreach((GList *)caller_data->linkhdr, 
		(gpointer)AddMenuItem, (gpointer)&menus);

	menubar = gtk_menu_bar_new();
	gtk_widget_set_name (menubar, "menu_bar");
	gtk_box_pack_start(GTK_BOX(inside_form), menubar, TRUE, TRUE, 0);

	/* Add the menus to the menu bar */
	for (i=0; i<4; i++)
	{
		if ( menus.widget[i] != NULL )
		{
			found = 1;
			button = gtk_menu_item_new_with_label(Menu_Labels[i]);
			gtk_widget_set_name (button, "main_menus");
			gtk_widget_show(button);
			gtk_menu_item_set_submenu(GTK_MENU_ITEM (button), menus.widget[i]);
			gtk_menu_bar_append(GTK_MENU_BAR (menubar), button);
		}
	}

	/* Don't show the menu bar if there was nothing in it */
	if ( found ) gtk_widget_show(menubar);
	return;
}

static void
AddMenuItem(
	GMenuData_T		*menudata,
	GMenus_T			*menus
)
{
	GtkWidget	*menu_item;

	/* Test if the parent menu has been created. If not, allocate it. */
	if ( menus->widget[menudata->menu_id] == NULL )
		menus->widget[menudata->menu_id] = gtk_menu_new();

	/* Create a menu entry. */
	menudata->widget = menu_item = gtk_menu_item_new_with_label(menudata->label);
	gtk_widget_set_name (menu_item, "main_menus");
	gtk_widget_show(menu_item);
	gtk_menu_append(GTK_MENU (menus->widget[menudata->menu_id]), menu_item);
	gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
		GTK_SIGNAL_FUNC(menudata->cb), (gpointer)NULL);
}
