/* $Id: merge.c,v 1.1.1.1 2004/10/11 17:35:33 mjhammel Exp $ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2001
 *========================================================================
 *              FILE NAME: merge.c
 *            DESCRIPTION: merge notes for XNotesPlus 
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPMergeNotes
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES:
 *	
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log: merge.c,v $
 * Revision 1.1.1.1  2004/10/11 17:35:33  mjhammel
 * Initial import
 *
 * Revision 1.1.1.1  2002/01/10 18:38:05  mjhammel
 * Initial checkin
 *
 *
 *========================================================================*/
#define XP_MERGE_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

/* GTK/GDK specific includes */
#include <gtk/gtk.h>
#include <gdk/gdkx.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void MergeNotesCB();


/* === Global Variables === */
extern AppRes		app_res;
extern StickyNote	*notes;
extern GtkWidget	*toplevel;


/* === Static Variables === */
static int	noteid;



/*========================================================================
 *	Name:			XPMergeNotes
 *	Prototype:	XPMergeNotes(int index)
 *					
 *	Description:
 * Prints a note.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPMergeNotes(
	int			index
)
{
	noteid = -1;
   XPGenericNoteList(MergeNoteList, FALSE,
      GTK_SELECTION_MULTIPLE, &MergeNotesCB, index);
}

static void
MergeNotesCB(
	GtkWidget		*widget,
	gpointer			index
)
{
	GtkWidget		*list_item;
	StickyNote		*pn;
	StickyNote		*newpn;
	char				*note;
	int				count;
	int				selection_length;
	int				position;

	/*
	 * We'll get called twice as a callback.  The first one to pass in the
	 * note index and the second to begin processing note merging.
	 */
	if (noteid == -1)
	{
		noteid = (int)index;
		return;
	}

	/* 
	 * For each selected note, append it's contents to the note that 
	 * requested the merging.
	 */
	if ((pn = XPFindNote(noteid)) == NULL)
		return;

	selection_length = g_list_length(GTK_LIST(index)->selection);

	for ( count=selection_length-1; count > -1; count-- )
	{
		list_item=GTK_WIDGET(g_list_nth_data(GTK_LIST(index)->selection, count));
		newpn = (StickyNote *) gtk_object_get_user_data(GTK_OBJECT(list_item));

		/* Skip merging if user selected the current note. */
		if (newpn == pn)
			continue;

		/* Get the insertion point - the end of the current text. */
		position = gtk_text_get_point (GTK_TEXT (pn->pn_textwidget));

		/* Get the text to insert. */
		note = gtk_editable_get_chars(GTK_EDITABLE(newpn->pn_textwidget), 0, -1);

		/*
		 * Now stuff the merged note in at the insertion point.
		 * Prefix and suffix it with newlines for clarity.
		 */
		gtk_text_freeze (GTK_TEXT (pn->pn_textwidget));
		gtk_text_insert (
			GTK_TEXT (pn->pn_textwidget),
			NULL, 					/* font */
			&(pn->pn_textwidget)->style->black, /* fg color */
			NULL,						/* bg color */
			"\n", 					/* text to insert */
			-1);						/* -1: use strlen(); >0: #chars to insert */
		gtk_text_insert (
			GTK_TEXT (pn->pn_textwidget),
			NULL, 					/* font */
			&(pn->pn_textwidget)->style->black, /* fg color */
			NULL,						/* bg color */
			note, 					/* text to insert */
			-1);						/* -1: use strlen(); >0: #chars to insert */
		gtk_text_insert (
			GTK_TEXT (pn->pn_textwidget),
			NULL, 					/* font */
			&(pn->pn_textwidget)->style->black, /* fg color */
			NULL,						/* bg color */
			"\n", 					/* text to insert */
			-1);						/* -1: use strlen(); >0: #chars to insert */
		gtk_text_thaw (GTK_TEXT (pn->pn_textwidget));
		gtk_text_set_point (GTK_TEXT (pn->pn_textwidget), 
			position + strlen(note) + 2);
		g_free(note);
	}
}

